import 'dart:io';

import 'package:flutter/material.dart';
import 'package:foundation_fluttify/foundation_fluttify.dart';
import 'package:tencent_live_fluttify/src/android/android.export.g.dart';
import 'package:tencent_live_fluttify/src/facade/tencent_live_controller.dart';
import 'package:tencent_live_fluttify/src/ios/ios.export.g.dart';

typedef Future<void> OnLiveWidgetCreated(TencentLiveController controller);

class TencentLiveWidget extends StatelessWidget {
  const TencentLiveWidget({
    Key key,
    this.enableTouchFocus,
    this.onLiveWidgetCreated,
  }) : super(key: key);

  /// 手动对焦还是自动对焦
  final bool enableTouchFocus;

  /// widget创建完成回调
  final OnLiveWidgetCreated onLiveWidgetCreated;

  @override
  Widget build(BuildContext context) {
    if (Platform.isAndroid) {
      return com_tencent_rtmp_ui_TXCloudVideoView_Android(
        onViewCreated: (controller) async {
          final context = await android_app_Application.get();

          final config = await com_tencent_rtmp_TXLivePushConfig.create__();
          if (enableTouchFocus != null) {
            await config.setTouchFocus(enableTouchFocus);
          }

          final pusher = await com_tencent_rtmp_TXLivePusher
              .create__android_content_Context(context);

          await pusher.setConfig(config);

          await pusher.startCameraPreview(controller);

          if (OnLiveWidgetCreated != null) {
            onLiveWidgetCreated(TencentLiveController.android(pusher, config));
          }
        },
      );
    } else if (Platform.isIOS) {
      return UIViewWidget(
        onUIViewCreated: (controller) async {
          final config = await TXLivePushConfig.create__();
          if (enableTouchFocus != null) {
            await config.set_touchFocus(enableTouchFocus);
          }

          final pusher = await TXLivePush.create__();
          await pusher.initWithConfig(config);

          await pusher.startPreview(controller);

          if (OnLiveWidgetCreated != null) {
            onLiveWidgetCreated(TencentLiveController.ios(pusher, config));
          }
        },
      );
    } else {
      return Center(
        child: Text('未实现的平台'),
      );
    }
  }
}
